#include "libcardfile.h"

/*******************************************************************
 * GetFileType
 * -----------------------------------------------------------------
 * Verifica si el archivo ingresado es un archivo compatible
 *
 * Retorna: (0) si es un archivo MGC, (1) si es RRG, (2) si es DKO.
 * Retorna errores: (-2) si no se pudo abrir el archivo, (-3) si el 
 * archivo no es compatible.
 *******************************************************************/
int GetFileType (FILE *fileptr)
{
	CARDFILEHEADER crdhdr;

	if (!fileptr)
		return ERROR_OPENFILE_FAILED;
	/* Lee los 3 primeros bytes en busca del nmero mgico y verifica si coincide con alguno de ellos */
	fread (crdhdr.magic_number, sizeof (crdhdr.magic_number), 1, fileptr);
	if (memcmp (crdhdr.magic_number, header_MGC, sizeof (crdhdr.magic_number)) == 0) {
		return 0;
	}
	else if (memcmp (crdhdr.magic_number, header_RRG, sizeof (crdhdr.magic_number)) == 0) {
		return 1;
	}
	else if (memcmp (crdhdr.magic_number, header_DKO, sizeof (crdhdr.magic_number)) == 0) {
		return 2;
	}
	else {/* El archivo no es compatible */
		return ERROR_UNSUPPORTED_FILETYPE;
	}
}

/*******************************************************************
* GetLastObjectId
* -----------------------------------------------------------------
* En las versiones RRG y DKO, obtiene el valor de los bytes 4 a 7 
* del archivo en el que se aloja este valor, para luego trabajar con 
* l, a pesar de que este valor no es de mucha utilidad, ms que la de 
* aparentemente servir como gua para aadir un nuevo ID de objeto.
*
* Retorna: el ID del ltimo objeto.
* Retorna error: (-2) si no se pudo abrir el archivo, (-3) si el tipo 
* de archivo no es compatible, (-5) si esta caracterstica no es 
* compatible con la versin del tipo de archivo (es decir, los MGC).
*******************************************************************/
int32_t GetLastObjectId (FILE *fileptr)
{
	CARDFILEHEADER crdhdr;
	int32_t id;

	if (!fileptr)
		return ERROR_OPENFILE_FAILED;
	/* Verifica si el tipo de archivo es compatible, y si la versin del
	   tipo de archivo es compatible: solo RRG y DKO.                     */
	switch (GetFileType (fileptr))
	{
		case 0:
			return ERROR_UNSUPPORTED_FEATURE;
		case 1:
		case 2:
			fseek (fileptr, sizeof (crdhdr.magic_number), SEEK_SET);
			fread (&crdhdr.last_object, sizeof (crdhdr.last_object), 1, fileptr);
			id = /*SwapToBigEndian32bits*/ (crdhdr.last_object);
			break;
		default:
			return ERROR_UNSUPPORTED_FILETYPE;
	}

	rewind (fileptr);

	return id;
}

/*******************************************************************
* GetCardsCount
* -----------------------------------------------------------------
* Lee los 2 bytes que siguen a [nmero mgico=3 bytes] (formato MGC) o
* a [nmero mgico=3 bytes]+[ultimo id de objeto=4 bytes] (formatos 
* RRG y DKO), para obtener de all el nmero de tarjetas (en orden 
* "little endian").
* 
* Retorna: el nmero de tarjetas.
* Retorna error: (-2) si no se pudo abrir el archivo, (-3) si el tipo
* de archivo no es compatible.
*******************************************************************/
int GetCardsCount (FILE *fileptr)
{
	CARDFILEHEADER crdhdr;

	int cards_number, offset;

	if (!fileptr)
		return ERROR_OPENFILE_FAILED;
	/* Verifica la versin del archivo para saber cunto moverse
	   y guarda ese valor en la variable para indicarla a la funcin */
	switch (GetFileType (fileptr))
	{
		case 0:
			offset = 3;
			break;
		case 1:
			offset = 7;
			break;
		case 2:
			printf ("[ERROR] DKO type cardfiles are not currently supported.\n");
			return ERROR_UNSUPPORTED_FILETYPE;
		default:
			return ERROR_UNSUPPORTED_FILETYPE;
	}
	/* Se mueve a la posicin indicada */
	fseek (fileptr, offset, SEEK_SET);
	/* Lee el valor y lo convierte a "big endian" para poder trabajar con l */
	fread (&crdhdr.number_cards, sizeof(crdhdr.number_cards), 1, fileptr);
	cards_number = SwapToBigEndian16bits (crdhdr.number_cards);
	/* Cierra todo y retorna el nmero de tarjetas */
	rewind (fileptr);

	return cards_number;
}

/*******************************************************************
* LoadCardNumberInfo
* -----------------------------------------------------------------
* Carga la informacin del ndice del archivo para poder trabajar con
* asus datos: la posicin u "offset" en la que comienzan los datos del 
* nmero de tarjeta indicado, y el ttulo de la tarjeta o "index".
* 
* Se debe leer secuencialmente el archivo en busca de la tarjeta 
* indicada desde la primera hasta la deseada, ya que no hay una 
* referencia a la ubicacin de la entrada de ndice.
*
* Retorna: un puntero a la estructura con la informacin obtenida.
* Retorna error: un puntero nulo en caso de error, que deber 
* controlarse y manejar excepciones al llamar la funcin.
*******************************************************************/
PINDEX LoadCardNumberInfo (FILE *fileptr, int card_number)
{
	PINDEX pindex;
	//CARDFILEINDEX index;
	//pindex = &index;
	pindex = malloc (sizeof (&pindex));

	if (GetFileType (fileptr) == 0)
		fseek (fileptr, 5, SEEK_SET);
	else if (GetFileType (fileptr) == 1 || GetFileType (fileptr) == 2)
		fseek (fileptr, 9, SEEK_SET);
	else 
		return NULL;

	for (int i = 0; i < card_number; i++)
	{
		fseek (fileptr, sizeof (pindex->reserved), SEEK_CUR);
		fread (pindex->card_data_pos, sizeof (pindex->card_data_pos), 1, fileptr);
		//SwapToBigEndian16bits (index->card_data_pos);

		fseek (fileptr, sizeof (pindex->flag), SEEK_CUR);
		fread (pindex->index_title, sizeof (pindex->index_title), 1, fileptr);
		fseek (fileptr, sizeof (pindex->end_flag), SEEK_CUR);
	}
	rewind (fileptr);
	free (&pindex);

	return pindex;
}

		